@@ -1,13 +1,13 @@
-# pylint:disable=too-many-lines
+# pylint:disable=too-many-lines, too-many-arguments
 """Defines spatial extent of objects."""
 
 from abc import ABC, abstractmethod
-from typing import List, Tuple, Union, Any
+from typing import List, Tuple, Union, Any, Callable
 
 import pydantic
 import numpy as np
 
-from shapely.geometry import Point, Polygon, box
+from shapely.geometry import Point, Polygon, box, MultiPolygon
 from descartes import PolygonPatch
 
 from .base import Tidy3dBaseModel
@@ -15,6 +15,7 @@ from .types import Bound, Size, Coordinate, Axis, Coordinate2D, tidynumpy, Array
 from .types import Vertices, Ax, Shapely
 from .viz import add_ax_if_none, equal_aspect
 from .viz import PLOT_BUFFER, ARROW_LENGTH_FACTOR, ARROW_WIDTH_FACTOR, MAX_ARROW_WIDTH_FACTOR
+from .viz import PlotParams, plot_params_geometry
 from ..log import Tidy3dKeyError, SetupError, ValidationError
 from ..constants import MICROMETER, LARGE_NUMBER, RADIAN
 
@@ -26,6 +27,11 @@ _N_SAMPLE_POLYGON_INTERSECT = 100
 class Geometry(Tidy3dBaseModel, ABC):
     """Abstract base class, defines where something exists in space."""
 
+    @property
+    def plot_params(self):
+        """Default parameters for plotting a Geometry object."""
+        return plot_params_geometry
+
     center: Coordinate = pydantic.Field(
         (0.0, 0.0, 0.0),
         title="Center",
@@ -195,7 +201,6 @@ class Geometry(Tidy3dBaseModel, ABC):
     def plot(
         self, x: float = None, y: float = None, z: float = None, ax: Ax = None, **patch_kwargs
     ) -> Ax:
-        # pylint:disable=line-too-long
         """Plot geometry cross section at single (x,y,z) coordinate.
 
         Parameters
@@ -218,17 +223,16 @@ class Geometry(Tidy3dBaseModel, ABC):
         matplotlib.axes._subplots.Axes
             The supplied or created matplotlib axes.
         """
-        # pylint:disable=line-too-long
 
         # find shapes that intersect self at plane
         axis, position = self.parse_xyz_kwargs(x=x, y=y, z=z)
         shapes_intersect = self.intersections(x=x, y=y, z=z)
 
+        plot_params = self.plot_params.include_kwargs(**patch_kwargs)
+
         # for each intersection, plot the shape
         for shape in shapes_intersect:
-            _shape = self.evaluate_inf_shape(shape)
-            patch = PolygonPatch(_shape, **patch_kwargs)
-            ax.add_artist(patch)
+            ax = self.plot_shape(shape, plot_params=plot_params, ax=ax)
 
         # clean up the axis display
         ax = self.add_ax_labels_lims(axis=axis, ax=ax)
@@ -236,6 +240,78 @@ class Geometry(Tidy3dBaseModel, ABC):
         ax.set_title(f"cross section at {'xyz'[axis]}={position:.2f}")
         return ax
 
+    def plot_shape(self, shape: Shapely, plot_params: PlotParams, ax: Ax) -> Ax:
+        """Defines how a shape is plotted on a matplotlib axes."""
+        _shape = self.evaluate_inf_shape(shape)
+        patch = PolygonPatch(_shape, **plot_params.to_kwargs())
+        ax.add_artist(patch)
+        return ax
+
+    @classmethod
+    def strip_coords(
+        cls, shape: Shapely
+    ) -> Tuple[List[float], List[float], Tuple[List[float], List[float]]]:
+        """Get the exterior and list of interior xy coords for a shape.
+
+        Parameters
+        ----------
+        shape: shapely.geometry.base.BaseGeometry
+            The shape that you want to strip coordinates from.
+
+        Returns
+        -------
+        Tuple[List[float], List[float], Tuple[List[float], List[float]]]
+            List of exterior xy coordinates
+            and a list of lists of the interior xy coordinates of the "holes" in the shape.
+        """
+
+        if isinstance(shape, Polygon):
+            ext_coords = shape.exterior.coords[:]
+            list_int_coords = [interior.coords[:] for interior in shape.interiors]
+        elif isinstance(shape, MultiPolygon):
+            all_ext_coords = []
+            list_all_int_coords = []
+            for _shape in shape.geoms:
+                all_ext_coords.append(_shape.exterior.coords[:])
+                all_int_coords = [_interior.coords[:] for _interior in _shape.interiors]
+                list_all_int_coords.append(all_int_coords)
+            ext_coords = np.concatenate(all_ext_coords, axis=0)
+            list_int_coords = [
+                np.concatenate(all_int_coords, axis=0)
+                for all_int_coords in list_all_int_coords
+                if len(all_int_coords) > 0
+            ]
+        return ext_coords, list_int_coords
+
+    @classmethod
+    def map_to_coords(cls, func: Callable[[float], float], shape: Shapely) -> Shapely:
+        """Maps a function to each coordinate in shape.
+
+        Parameters
+        ----------
+        func : Callable[[float], float]
+            Takes old coordinate and returns new coordinate.
+        shape: shapely.geometry.base.BaseGeometry
+            The shape to map this function to.
+
+        Returns
+        -------
+        shapely.geometry.base.BaseGeometry
+            A new copy of the input shape with the mapping applied to the coordinates.
+        """
+
+        if not isinstance(shape, (Polygon, MultiPolygon)):
+            return shape
+
+        def apply_func(coords):
+            return [(func(coord_x), func(coord_y)) for (coord_x, coord_y) in coords]
+
+        ext_coords, list_int_coords = cls.strip_coords(shape)
+        new_ext_coords = apply_func(ext_coords)
+        list_new_int_coords = [apply_func(int_coords) for int_coords in list_int_coords]
+
+        return Polygon(new_ext_coords, holes=list_new_int_coords)
+
     def _get_plot_labels(self, axis: Axis) -> Tuple[str, str]:
         """Returns planar coordinate x and y axis labels for cross section plots.
 
@@ -293,7 +369,7 @@ class Geometry(Tidy3dBaseModel, ABC):
         (xmin, xmax), (ymin, ymax) = self._get_plot_limits(axis=axis, buffer=buffer)
 
         # note: axes limits dont like inf values, so we need to evaluate them first if present
-        xmin, xmax, ymin, ymax = self._evaluate_infs(xmin, xmax, ymin, ymax)
+        xmin, xmax, ymin, ymax = (self._evaluate_inf(v) for v in (xmin, xmax, ymin, ymax))
 
         ax.set_xlim(xmin, xmax)
         ax.set_ylim(ymin, ymax)
@@ -302,23 +378,18 @@ class Geometry(Tidy3dBaseModel, ABC):
         return ax
 
     @staticmethod
-    def _evaluate_infs(*values):
+    def _evaluate_inf(v):
         """Processes values and evaluates any infs into large (signed) numbers."""
-        return map(lambda v: v if not np.isinf(v) else np.sign(v) * LARGE_NUMBER, values)
+        return v if not np.isinf(v) else np.sign(v) * LARGE_NUMBER
 
     @classmethod
-    def evaluate_inf_shape(cls, shape: "shapely.Geometry") -> "shapely.Geometry":
+    def evaluate_inf_shape(cls, shape: Shapely) -> Shapely:
         """Returns a copy of shape with inf vertices replaced by large numbers if polygon."""
 
         if not isinstance(shape, Polygon):
             return shape
 
-        coords = shape.exterior.coords[:]
-        new_coords = []
-        for (coord_x, coord_y) in coords:
-            new_coord = tuple(cls._evaluate_infs(coord_x, coord_y))
-            new_coords.append(new_coord)
-        return Polygon(new_coords)
+        return cls.map_to_coords(cls._evaluate_inf, shape)
 
     @staticmethod
     def pop_axis(coord: Tuple[Any, Any, Any], axis: int) -> Tuple[Any, Tuple[Any, Any]]:
@@ -644,7 +715,7 @@ class Box(Geometry):
     )
 
     @classmethod
-    def from_bounds(cls, rmin: Coordinate, rmax: Coordinate):
+    def from_bounds(cls, rmin: Coordinate, rmax: Coordinate, **kwargs):
         """Constructs a :class:`Box` from minimum and maximum coordinate bounds
 
         Parameters
@@ -663,11 +734,20 @@ class Box(Geometry):
             """Returns center point based on bounds along dimension."""
             if np.isneginf(pt_min) and np.isposinf(pt_max):
                 return 0.0
+            if np.isneginf(pt_min) or np.isposinf(pt_max):
+                raise SetupError(
+                    f"Bounds of ({pt_min}, {pt_max}) supplied along one dimension. "
+                    "We currently don't support a single ``inf`` value in bounds for ``Box``. "
+                    "To construct a semi-infinite ``Box``, "
+                    "please supply a large enough number instead of ``inf``. "
+                    "For example, a location extending outside of the "
+                    "Simulation domain (including PML)."
+                )
             return (pt_min + pt_max) / 2.0
 
         center = tuple(get_center(pt_min, pt_max) for pt_min, pt_max in zip(rmin, rmax))
         size = tuple((pt_max - pt_min) for pt_min, pt_max in zip(rmin, rmax))
-        return cls(center=center, size=size)
+        return cls(center=center, size=size, **kwargs)
 
     def intersections(self, x: float = None, y: float = None, z: float = None):
         """Returns shapely geometry at plane specified by one non None value of x,y,z.
@@ -792,8 +872,13 @@ class Box(Geometry):
         arrow_axis = [component == 0 for component in direction]
         arrow_length, arrow_width = self._arrow_dims(ax, length_factor, width_factor)
 
-        # only add arrow if the plotting plane is perpendicular to the source
-        if arrow_axis.count(0.0) > 1 or arrow_axis.index(0.0) != plot_axis:
+        # conditions to check to determine whether to plot arrow
+        arrow_intersecting_plane = len(self.intersections(x=x, y=y, z=z)) > 0
+        arrow_perp_to_screen = arrow_axis.index(0.0) != plot_axis
+        arrow_not_cartesian_axis = arrow_axis.count(0.0) > 1
+
+        # plot if arrow in plotting plane and some non-zero component can be displayed.
+        if arrow_intersecting_plane and (arrow_not_cartesian_axis or arrow_perp_to_screen):
             _, (x0, y0) = self.pop_axis(self.center, axis=plot_axis)
             _, (dx, dy) = self.pop_axis(direction, axis=plot_axis)
 
@@ -1214,7 +1299,7 @@ class PolySlab(Planar):
         return val
 
     @classmethod
-    def from_gds(  # pylint:disable=too-many-arguments
+    def from_gds(  # pylint:disable=too-many-arguments, too-many-locals
         cls,
         gds_cell,
         axis: Axis,
@@ -1224,6 +1309,7 @@ class PolySlab(Planar):
         gds_scale: pydantic.PositiveFloat = 1.0,
         dilation: float = 0.0,
         sidewall_angle: float = 0,
+        **kwargs,
     ) -> List["PolySlab"]:
         """Import :class:`PolySlab` from a ``gdspy.Cell``.
 
@@ -1285,6 +1371,7 @@ class PolySlab(Planar):
                 slab_bounds=slab_bounds,
                 dilation=dilation,
                 sidewall_angle=sidewall_angle,
+                **kwargs,
             )
             for verts in all_vertices
         ]
@@ -1454,12 +1541,19 @@ class PolySlab(Planar):
         polys = []
 
         # looping through z_i to assemble the polygons
-        height = 0.0
         height_list = np.append(height_list, self.length)
-        for h_local in height_list:
-            dist = -height * self._tanq
+        h_base = 0.0
+        for h_top in height_list:
+            # length within between top and bottom
+            h_length = h_top - h_base
+
+            # coordinate of each subsection
+            z_min, z_max = z_base + h_base, z_base + h_top
+
+            # vertices for the base of each subsection
+            dist = -h_base * self._tanq
             vertices = self._shift_vertices(self.base_polygon, dist)[0]
-            z_min, z_max = z_base + height, z_base + height + h_local
+
             # for vertical sidewall, no need for complications
             if np.isclose(self.sidewall_angle, 0):
                 ints_y, ints_angle = self._find_intersecting_ys_angle_vertical(
@@ -1486,8 +1580,8 @@ class PolySlab(Planar):
                     angle_min = np.arctan(np.tan(self.sidewall_angle) / np.sin(angle_min))
                     angle_max = np.arctan(np.tan(self.sidewall_angle) / np.sin(angle_max))
 
-                    dy_min = h_local * np.tan(angle_min)
-                    dy_max = h_local * np.tan(angle_max)
+                    dy_min = h_length * np.tan(angle_min)
+                    dy_max = h_length * np.tan(angle_max)
 
                     x1, y1 = self._order_by_axis(plane_val=y_min, axis_val=z_min, axis=axis)
                     x2, y2 = self._order_by_axis(plane_val=y_max, axis_val=z_min, axis=axis)
@@ -1495,9 +1589,9 @@ class PolySlab(Planar):
                     if y_max - y_min <= dy_min + dy_max:
                         # intersect before reaching top of polygon
                         # make triangle
-                        h_mid = (y_max - y_min) / (dy_min + dy_max) * h_local
+                        h_mid = (y_max - y_min) / (dy_min + dy_max) * h_length
                         z_mid = z_min + h_mid
-                        y_mid = y_min + dy_min / h_local * h_mid
+                        y_mid = y_min + dy_min / h_length * h_mid
                         x3, y3 = self._order_by_axis(plane_val=y_mid, axis_val=z_mid, axis=axis)
                         vertices = ((x1, y1), (x2, y2), (x3, y3))
                         polys.append(Polygon(vertices))
@@ -1512,7 +1606,8 @@ class PolySlab(Planar):
                         vertices = ((x1, y1), (x2, y2), (x3, y3), (x4, y4))
                         polys.append(Polygon(vertices))
 
-            height += h_local
+            # update the base coordinate for the next subsection
+            h_base = h_top
 
         return polys
 
@@ -1555,7 +1650,7 @@ class PolySlab(Planar):
         return height
 
     def _find_intersecting_ys_angle_vertical(  # pylint:disable=too-many-locals
-        self, vertices: np.ndarray, position: float, axis: int
+        self, vertices: np.ndarray, position: float, axis: int, exclude_on_vertices: bool = False
     ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
         """Finds pairs of forward and backwards vertices where polygon intersects position at axis,
         Find intersection point (in y) assuming straight line,and intersecting angle between plane
@@ -1570,6 +1665,8 @@ class PolySlab(Planar):
             position along axis.
         axis : int
             Integer index into 'xyz' (0,1,2).
+        exclude_on_vertices : bool = False
+            Whehter to exclude those intersecting directly with the vertices.
 
         Returns
         -------
@@ -1586,13 +1683,23 @@ class PolySlab(Planar):
         # get the forward vertices
         vertices_f = np.roll(vertices_axis, shift=-1, axis=0)
 
+        # x coordinate of the two sets of vertices
+        x_vertices_f = vertices_f[:, 0]
+        x_vertices_axis = vertices_axis[:, 0]
+
         # find which segments intersect
-        intersects_b = np.logical_and(
-            (vertices_f[:, 0] <= position), (vertices_axis[:, 0] > position)
+        f_left_to_intersect = (
+            x_vertices_f < position if exclude_on_vertices else x_vertices_f <= position
         )
-        intersects_f = np.logical_and(
-            (vertices_axis[:, 0] <= position), (vertices_f[:, 0] > position)
+        orig_right_to_intersect = x_vertices_axis > position
+        intersects_b = np.logical_and(f_left_to_intersect, orig_right_to_intersect)
+
+        f_right_to_intersect = x_vertices_f > position
+        orig_left_to_intersect = (
+            x_vertices_axis < position if exclude_on_vertices else x_vertices_axis <= position
         )
+        intersects_f = np.logical_and(f_right_to_intersect, orig_left_to_intersect)
+
         intersects_segment = np.logical_or(intersects_b, intersects_f)
         iverts_b = vertices_axis[intersects_segment]
         iverts_f = vertices_f[intersects_segment]
@@ -1652,7 +1759,9 @@ class PolySlab(Planar):
         vertices_b = np.roll(vertices_axis, shift=1, axis=0)
 
         ## First part, plane intersects with edges, same as vertical
-        ints_y, ints_angle = self._find_intersecting_ys_angle_vertical(vertices, position, axis)
+        ints_y, ints_angle = self._find_intersecting_ys_angle_vertical(
+            vertices, position, axis, exclude_on_vertices=True
+        )
         ints_y = ints_y.tolist()
         ints_angle = ints_angle.tolist()
 
@@ -1699,10 +1808,16 @@ class PolySlab(Planar):
                 # the vertex will split into two
                 ints_y.append(y_on)
                 ints_y.append(y_on)
-                slope = (y_on - y_b) / (x_on - x_b)
-                ints_angle.append(np.pi / 2 - np.arctan(np.abs(slope)))
-                slope = (y_on - y_f) / (x_on - x_f)
-                ints_angle.append(np.pi / 2 - np.arctan(np.abs(slope)))
+
+                # the order of the two new vertices needs to handled correctly;
+                # it should be sorted according to the -slope * moving direction
+                slope = [(y_on - y_b) / (x_on - x_b), (y_on - y_f) / (x_on - x_f)]
+                dressed_slope = [-s_i * shift_local for s_i in slope]
+                sort_index = np.argsort(np.array(dressed_slope))
+                sorted_slope = np.array(slope)[sort_index]
+
+                ints_angle.append(np.pi / 2 - np.arctan(np.abs(sorted_slope[0])))
+                ints_angle.append(np.pi / 2 - np.arctan(np.abs(sorted_slope[1])))
             # case 3, one of neightbouring vertices on the plane too:
             else:
                 # now only for angle<np.pi/2, so it's not relevant
