@@ -4,16 +4,16 @@ from abc import ABC, abstractmethod
 from typing import Union, Tuple
 import logging
 
-from typing_extensions import Annotated
+from typing_extensions import Literal
 import pydantic
 import numpy as np
 
-from .base import Tidy3dBaseModel, TYPE_TAG_STR
-from .types import Direction, Polarization, Ax, FreqBound, Array, Axis
+from .base import Tidy3dBaseModel
+from .types import Direction, Polarization, Ax, FreqBound, Array, Axis, ArrayLike
 from .validators import assert_plane, validate_name_str
 from .geometry import Box
 from .mode import ModeSpec
-from .viz import add_ax_if_none, SourceParams, equal_aspect
+from .viz import add_ax_if_none, PlotParams, plot_params_source
 from .viz import ARROW_COLOR_SOURCE, ARROW_ALPHA, ARROW_COLOR_POLARIZATION
 from ..constants import RADIAN, HERTZ, MICROMETER, GLANCING_CUTOFF
 from ..constants import inf  # pylint:disable=unused-import
@@ -260,33 +260,74 @@ class Source(Box, ABC):
 
     name: str = pydantic.Field(None, title="Name", description="Optional name for the source.")
 
+    @property
+    def plot_params(self) -> PlotParams:
+        """Default parameters for plotting a Source object."""
+        return plot_params_source
+
     _name_validator = validate_name_str()
 
-    @equal_aspect
-    @add_ax_if_none
-    def plot(
-        self, x: float = None, y: float = None, z: float = None, ax: Ax = None, **kwargs
-    ) -> Ax:
-
-        kwargs = SourceParams().update_params(**kwargs)
-        ax = self.geometry.plot(x=x, y=y, z=z, ax=ax, **kwargs)
-        return ax
-
     @property
-    def geometry(self):
+    def geometry(self) -> Box:
         """:class:`Box` representation of source."""
 
         return Box(center=self.center, size=self.size)
 
+    @property
+    def _dir_vector(self) -> Tuple[float, float, float]:
+        """Returns a vector indicating the source direction for arrow plotting, if not None."""
+        return None
 
-class VolumeSource(Source):
-    """Source in a rectangular volume with uniform time dependence. size=(0,0,0) gives point source.
+    @property
+    def _pol_vector(self) -> Tuple[float, float, float]:
+        """Returns a vector indicating the source polarization for arrow plotting, if not None."""
+        return None
 
-    Example
-    -------
-    >>> pulse = GaussianPulse(freq0=200e12, fwidth=20e12)
-    >>> pt_source = VolumeSource(size=(0,0,0), source_time=pulse, polarization='Ex')
-    """
+    def plot(
+        self, x: float = None, y: float = None, z: float = None, ax: Ax = None, **kwargs
+    ) -> Ax:
+
+        # call the `Source.plot()` function first.
+        ax = super().plot(x=x, y=y, z=z, ax=ax, **kwargs)
+
+        kwargs_alpha = kwargs.get("alpha")
+        arrow_alpha = ARROW_ALPHA if kwargs_alpha is None else kwargs_alpha
+
+        # then add the arrow based on the propagation direction
+        if self._dir_vector is not None:
+
+            ax = self._plot_arrow(
+                x=x,
+                y=y,
+                z=z,
+                ax=ax,
+                direction=self._dir_vector,
+                color=ARROW_COLOR_SOURCE,
+                alpha=arrow_alpha,
+                both_dirs=False,
+            )
+
+        if self._pol_vector is not None:
+
+            ax = self._plot_arrow(
+                x=x,
+                y=y,
+                z=z,
+                ax=ax,
+                direction=self._pol_vector,
+                color=ARROW_COLOR_POLARIZATION,
+                alpha=arrow_alpha,
+                both_dirs=False,
+            )
+
+        return ax
+
+
+""" Sources either: (1) implement current distributions or (2) generate fields."""
+
+
+class CurrentSource(Source, ABC):
+    """Source implements a current distribution directly."""
 
     polarization: Polarization = pydantic.Field(
         ...,
@@ -294,6 +335,59 @@ class VolumeSource(Source):
         description="Specifies the direction and type of current component.",
     )
 
+    @property
+    def _pol_vector(self) -> Tuple[float, float, float]:
+        """Returns a vector indicating the source polarization for arrow plotting, if not None."""
+        component = self.polarization[-1]  # 'x' 'y' or 'z'
+        if component in "xyz":
+            pol_axis = "xyz".index(component)
+            pol_vec = [0, 0, 0]
+            pol_vec[pol_axis] = 1
+            return pol_vec
+        return None
+
+
+class UniformCurrentSource(CurrentSource):
+    """Source in a rectangular volume with uniform time dependence. size=(0,0,0) gives point source.
+
+    Example
+    -------
+    >>> pulse = GaussianPulse(freq0=200e12, fwidth=20e12)
+    >>> pt_source = UniformCurrentSource(size=(0,0,0), source_time=pulse, polarization='Ex')
+    """
+
+
+class PointDipole(CurrentSource):
+    """Uniform current source with a zero size.
+
+    Example
+    -------
+    >>> pulse = GaussianPulse(freq0=200e12, fwidth=20e12)
+    >>> pt_dipole = PointDipole(center=(1,2,3), source_time=pulse, polarization='Ex')
+    """
+
+    size: Tuple[Literal[0], Literal[0], Literal[0]] = (0, 0, 0)
+
+
+class FieldSource(Source, ABC):
+    """A Source defined by the desired E and/or H fields."""
+
+
+""" TODO: Custom currents """
+
+
+class CustomSource(Source, ABC):
+    """Implements custom current components specified by data."""
+
+    data: ArrayLike
+
+
+class CustomFieldSource(FieldSource, CustomSource):
+    """Implements custom E, H fields specified by data."""
+
+
+""" Field Sources can be defined either on a (1) surface or (2) volume. Defines injection_axis """
+
 
 class PlanarSource(Source, ABC):
     """A source defined on a 2D plane."""
@@ -305,28 +399,9 @@ class PlanarSource(Source, ABC):
         """Injection axis of the source."""
         return self.size.index(0.0)
 
-    @injection_axis.setter
-    def injection_axis(self):
-        raise ValueError("PlanarSource injection axis is defined by the axis normal to the plane.")
 
-
-class FieldSource(Source, ABC):
-    """A Source defined by the desired E and/or H fields."""
-
-
-class DirectionalSource(FieldSource, ABC):
-    """A FieldSource defined with a direction of propagation. The direction is defined by
-    the polar and azimuth angles w.r.t. an injection axis, as well as forward ``+`` or
-    backward ``-``. This base class only defines the ``direction`` and ``injection_axis``
-    attributes, but it must be composed with a class that also defines ``angle_theta`` and
-    ``angle_phi``."""
-
-    direction: Direction = pydantic.Field(
-        ...,
-        title="Direction",
-        description="Specifies propagation in the positive or negative direction of the injection "
-        "axis.",
-    )
+class VolumeSource(Source, ABC):
+    """A source defined in a 3D :class:`Box`."""
 
     injection_axis: Axis = pydantic.Field(
         None,
@@ -336,29 +411,82 @@ class DirectionalSource(FieldSource, ABC):
         "directional sources, as it is taken automatically from the plane size.",
     )
 
-    def plot(
-        self, x: float = None, y: float = None, z: float = None, ax: Ax = None, **kwargs
-    ) -> Ax:
 
-        # call the `Source.plot()` function first.
-        ax = super().plot(x=x, y=y, z=z, ax=ax, **kwargs)
+""" Field Sources require more specification, for now, they all have a notion of a direction."""
 
-        # then add the arrow based on the propagation direction
-        ax = self._plot_arrow(
-            x=x,
-            y=y,
-            z=z,
-            ax=ax,
-            direction=self._dir_arrow,
-            color=ARROW_COLOR_SOURCE,
-            alpha=ARROW_ALPHA,
-            both_dirs=False,
-        )
-        return ax
+
+class DirectionalSource(FieldSource, ABC):
+    """A Field source that propagates in a given direction."""
+
+    direction: Direction = pydantic.Field(
+        ...,
+        title="Direction",
+        description="Specifies propagation in the positive or negative direction of the injection "
+        "axis.",
+    )
+
+    @property
+    def _dir_vector(self) -> Tuple[float, float, float]:
+        """Returns a vector indicating the source direction for arrow plotting, if not None."""
+        if hasattr(self, "injection_axis"):
+            dir_vec = [0, 0, 0]
+            dir_vec[self.injection_axis] = 1 if self.direction == "+" else -1
+            return dir_vec
+        return None
+
+
+""" Source current profiles defined by (1) angle or (2) desired mode. Sets theta and phi angles."""
+
+
+class AngledFieldSource(DirectionalSource, ABC):
+    """A FieldSource defined with a an angled direction of propagation. The direction is defined by
+    the polar and azimuth angles w.r.t. an injection axis, as well as forward ``+`` or
+    backward ``-``. This base class only defines the ``direction`` and ``injection_axis``
+    attributes, but it must be composed with a class that also defines ``angle_theta`` and
+    ``angle_phi``."""
+
+    angle_theta: float = pydantic.Field(
+        0.0,
+        title="Polar Angle",
+        description="Polar angle of the propagation axis from the injection axis.",
+        units=RADIAN,
+    )
+
+    angle_phi: float = pydantic.Field(
+        0.0,
+        title="Azimuth Angle",
+        description="Azimuth angle of the propagation axis in the plane orthogonal to the "
+        "injection axis.",
+        units=RADIAN,
+    )
+
+    pol_angle: float = pydantic.Field(
+        0,
+        title="Polarization Angle",
+        description="Specifies the angle between the electric field polarization of the "
+        "source and the plane defined by the injection axis and the propagation axis (rad). "
+        "``pol_angle=0`` (default) specifies P polarization, "
+        "while ``pol_angle=np.pi/2`` specifies S polarization. "
+        "At normal incidence when S and P are undefined, ``pol_angle=0`` defines: "
+        "- ``Ey`` polarization for propagation along ``x``."
+        "- ``Ex`` polarization for propagation along ``y``."
+        "- ``Ex`` polarization for propagation along ``z``.",
+        units=RADIAN,
+    )
+
+    @pydantic.validator("angle_theta", allow_reuse=True, always=True)
+    def glancing_incidence(cls, val):
+        """Warn if close to glancing incidence."""
+        if np.abs(np.pi / 2 - val) < GLANCING_CUTOFF:
+            logging.warning(
+                "Angled source propagation axis close to glancing angle. "
+                "For best results, switch the injection axis."
+            )
+        return val
 
     # pylint: disable=no-member
     @property
-    def _dir_arrow(self) -> Tuple[float, float, float]:
+    def _dir_vector(self) -> Tuple[float, float, float]:
         """Source direction normal vector in cartesian coordinates."""
         radius = 1.0 if self.direction == "+" else -1.0
         dx = radius * np.cos(self.angle_phi) * np.sin(self.angle_theta)
@@ -366,8 +494,19 @@ class DirectionalSource(FieldSource, ABC):
         dz = radius * np.cos(self.angle_theta)
         return self.unpop_axis(dz, (dx, dy), axis=self.injection_axis)
 
+    @property
+    def _pol_vector(self) -> Tuple[float, float, float]:
+        """Source polarization normal vector in cartesian coordinates."""
+        normal_dir = [0.0, 0.0, 0.0]
+        normal_dir[self.injection_axis] = 1.0
+        propagation_dir = list(self._dir_vector)
+        pol_vector = np.cross(normal_dir, propagation_dir)
+        if np.all(pol_vector == 0.0):
+            pol_vector = np.array((0, 1, 0)) if self.injection_axis == 0 else np.array((1, 0, 0))
+        return self.rotate_points(pol_vector, propagation_dir, angle=self.pol_angle)
 
-class ModeSource(PlanarSource, DirectionalSource):
+
+class ModeSource(DirectionalSource, PlanarSource):
     """Injects current source to excite modal profile on finite extent plane.
 
     Example
@@ -407,86 +546,20 @@ class ModeSource(PlanarSource, DirectionalSource):
         """Azimuth angle of propagation."""
         return self.mode_spec.angle_phi
 
-
-class AngledDirectionalSource(DirectionalSource, ABC):
-    """Directional source with explicitly defined ``angle_theta`` and ``angle_phi`` fields."""
-
-    angle_theta: float = pydantic.Field(
-        0.0,
-        title="Polar Angle",
-        description="Polar angle of the propagation axis from the injection axis.",
-        units=RADIAN,
-    )
-
-    angle_phi: float = pydantic.Field(
-        0.0,
-        title="Azimuth Angle",
-        description="Azimuth angle of the propagation axis in the plane orthogonal to the "
-        "injection axis.",
-        units=RADIAN,
-    )
-
-    @pydantic.validator("angle_theta", allow_reuse=True, always=True)
-    def glancing_incidence(cls, val):
-        """Warn if close to glancing incidence."""
-        if np.abs(np.pi / 2 - val) < GLANCING_CUTOFF:
-            logging.warning(
-                "Angled source propagation axis close to glancing angle. "
-                "For best results, switch the injection axis."
-            )
-        return val
-
-
-class PolarizedSource(AngledDirectionalSource, ABC):
-    """AngledDirectionalSource with a polarization angle."""
-
-    pol_angle: float = pydantic.Field(
-        0,
-        title="Polarization Angle",
-        description="Specifies the angle between the electric field polarization of the "
-        "source and the plane defined by the injection axis and the propagation axis (rad). "
-        "``pol_angle=0`` (default) specifies P polarization, "
-        "while ``pol_angle=np.pi/2`` specifies S polarization. "
-        "At normal incidence when S and P are undefined, ``pol_angle=0`` defines: "
-        "- ``Ey`` polarization for propagation along ``x``."
-        "- ``Ex`` polarization for propagation along ``y``."
-        "- ``Ex`` polarization for propagation along ``z``.",
-        units=RADIAN,
-    )
-
-    def plot(
-        self, x: float = None, y: float = None, z: float = None, ax: Ax = None, **kwargs
-    ) -> Ax:
-
-        # call the `DirectionalSource.plot()` function first (including dir arrow)
-        ax = super().plot(x=x, y=y, z=z, ax=ax, **kwargs)
-
-        # then add another arrow based on the polarization direction
-        ax = self._plot_arrow(
-            x=x,
-            y=y,
-            z=z,
-            ax=ax,
-            direction=self._pol_arrow,
-            color=ARROW_COLOR_POLARIZATION,
-            alpha=ARROW_ALPHA,
-            both_dirs=False,
-        )
-        return ax
-
     @property
-    def _pol_arrow(self) -> Tuple[float, float, float]:
-        """Source polarization normal vector in cartesian coordinates."""
-        normal_dir = [0.0, 0.0, 0.0]
-        normal_dir[self.injection_axis] = 1.0
-        propagation_dir = list(self._dir_arrow)
-        pol_vector = np.cross(normal_dir, propagation_dir)
-        if np.all(pol_vector == 0.0):
-            pol_vector = np.array((0, 1, 0)) if self.injection_axis == 0 else np.array((1, 0, 0))
-        return self.rotate_points(pol_vector, propagation_dir, angle=self.pol_angle)
+    def _dir_vector(self) -> Tuple[float, float, float]:
+        """Source direction normal vector in cartesian coordinates."""
+        radius = 1.0 if self.direction == "+" else -1.0
+        dx = radius * np.cos(self.angle_phi) * np.sin(self.angle_theta)
+        dy = radius * np.sin(self.angle_phi) * np.sin(self.angle_theta)
+        dz = radius * np.cos(self.angle_theta)
+        return self.unpop_axis(dz, (dx, dy), axis=self.injection_axis)
 
 
-class PlaneWave(PlanarSource, PolarizedSource):
+""" Angled Field Sources one can use. """
+
+
+class PlaneWave(AngledFieldSource, PlanarSource):
     """Uniform current distribution on an infinite extent plane.  One element of size must be zero.
 
     Example
@@ -506,7 +579,7 @@ class PlaneWave(PlanarSource, PolarizedSource):
         return val
 
 
-class GaussianBeam(PlanarSource, PolarizedSource):
+class GaussianBeam(AngledFieldSource, PlanarSource):
     """Guassian distribution on finite extent plane.
 
     Example
@@ -535,8 +608,9 @@ class GaussianBeam(PlanarSource, PolarizedSource):
     )
 
 
+class TFSF(AngledFieldSource, VolumeSource):
+    """Total field scattered field with a plane wave field in a volume."""
+
+
 # sources allowed in Simulation.sources
-SourceType = Annotated[
-    Union[VolumeSource, GaussianBeam, ModeSource, PlaneWave],
-    pydantic.Field(discriminator=TYPE_TAG_STR),
-]
+SourceType = Union[UniformCurrentSource, PointDipole, GaussianBeam, ModeSource, PlaneWave]
