@@ -45,7 +45,7 @@ PML_SIGMA = "2*EPSILON_0/dt"
 RADPERSEC = "rad/sec"
 
 # large number used for comparing infinity
-LARGE_NUMBER = np.float32(1e20)
+LARGE_NUMBER = 1e10
 
 inf = np.inf
 
