@@ -3,9 +3,7 @@ import os
 import getpass
 import hashlib
 import json
-import functools
 
-import boto3
 import requests
 
 from .config import DEFAULT_CONFIG as Config
@@ -19,8 +17,6 @@ CREDENTIAL_FILE = "~/.tidy3d/auth.json"
 credential_path = os.path.expanduser(CREDENTIAL_FILE)
 credential_dir = os.path.split(credential_path)[0]
 
-boto3.setup_default_session(region_name=Config.s3_region)
-
 
 def set_authentication_config(email: str, password: str) -> None:
     """Sets the authorization and keys in the config for a for user."""
@@ -59,7 +55,7 @@ def get_credentials() -> None:
     if "TIDY3D_USER" in os.environ and (
         "TIDY3D_PASS" in os.environ or "TIDY3D_PASS_HASH" in os.environ
     ):
-        log.debug("Using Tidy3D credentials from enviornment")
+        log.debug("Using Tidy3D credentials from environment")
         email = os.environ["TIDY3D_USER"]
         password = os.environ.get("TIDY3D_PASS")
         if password is None:
@@ -130,14 +126,3 @@ def get_credentials() -> None:
 
         # otherwise, prompt again
         log.info(f"Unknown response: {keep_logged_in}")
-
-
-def requires_auth(func):
-    """Decorator for functions that require the authentication step."""
-
-    @functools.wraps(func)
-    def auth_before_call(*args, **kwargs):
-        get_credentials()
-        return func(*args, **kwargs)
-
-    return auth_before_call
