@@ -4,8 +4,6 @@ from typing import Any
 
 from dataclasses import dataclass
 
-SOLVER_VERSION = "release-22.1.6"
-
 
 @dataclass
 class WebConfig:  # pylint:disable=too-many-instance-attributes
@@ -15,7 +13,8 @@ class WebConfig:  # pylint:disable=too-many-instance-attributes
     studio_bucket: str
     auth_api_endpoint: str
     web_api_endpoint: str
-    solver_version: str = SOLVER_VERSION
+    website_endpoint: str
+    solver_version: str = None
     worker_group: Any = None
     auth: str = None
     user: str = None
@@ -28,6 +27,7 @@ ConfigDev = WebConfig(
     studio_bucket="flow360-studio-v1",
     auth_api_endpoint="https://portal-api.dev-simulation.cloud",
     web_api_endpoint="https://tidy3d-api.dev-simulation.cloud",
+    website_endpoint="https://dev-tidy3d.simulation.cloud",
 )
 
 # staging config
@@ -36,6 +36,16 @@ ConfigUat = WebConfig(
     studio_bucket="flow360studio",
     auth_api_endpoint="https://portal-api.simulation.cloud",
     web_api_endpoint="https://uat-tidy3d-api.simulation.cloud",
+    website_endpoint="https://uat-tidy3d.simulation.cloud",
+)
+
+# pre-production config
+ConfigPreProd = WebConfig(
+    s3_region="us-gov-west-1",
+    studio_bucket="flow360studio",
+    auth_api_endpoint="https://preprod-portal-api.simulation.cloud",
+    web_api_endpoint="https://preprod-tidy3d-api.simulation.cloud",
+    website_endpoint="https://preprod-tidy3d.simulation.cloud",
 )
 
 # production config
@@ -44,6 +54,7 @@ ConfigProd = WebConfig(
     studio_bucket="flow360studio",
     auth_api_endpoint="https://portal-api.simulation.cloud",
     web_api_endpoint="https://tidy3d-api.simulation.cloud",
+    website_endpoint="https://tidy3d.simulation.cloud",
 )
 
 # default one to import
@@ -53,5 +64,7 @@ if os.environ.get("TIDY3D_ENV") == "dev":
     DEFAULT_CONFIG = ConfigDev
 elif os.environ.get("TIDY3D_ENV") == "uat":
     DEFAULT_CONFIG = ConfigUat
+elif os.environ.get("TIDY3D_ENV") == "preprod":
+    DEFAULT_CONFIG = ConfigPreProd
 elif os.environ.get("TIDY3D_ENV") == "prod":
     DEFAULT_CONFIG = ConfigProd
