@@ -1,11 +1,12 @@
 """higher level wrappers for webapi functions for individual (Job) and batch (Batch) tasks."""
 import os
 from abc import ABC
-from typing import Dict, Generator, Optional
+from typing import Dict, Optional, Tuple
 import time
 
 from rich.console import Console
 from rich.progress import Progress
+import pydantic as pd
 
 from . import webapi as web
 from .task import TaskId, TaskInfo, RunInfo, TaskName
@@ -23,26 +24,31 @@ class WebContainer(Tidy3dBaseModel, ABC):
 
 
 class Job(WebContainer):
-    """Interface for managing the running of a :class:`.Simulation` on server.
+    """Interface for managing the running of a :class:`.Simulation` on server."""
 
-    Parameters
-    ----------
-        simulation : :class:`.Simulation`
-            Simulation to upload to server.
-        task_name : str
-            Name of task.
-        folder_name : str = "default"
-            Name of folder to store task on web UI.
-        callback_url : str = None
-            Http PUT url to receive simulation finish event. The body content is a json file with
-            fields ``{'id', 'status', 'name', 'workUnit', 'solverVersion'}``.
-    """
+    simulation: Simulation = pd.Field(
+        ..., title="Simulation", description="Simulation to run as a 'task'."
+    )
 
-    simulation: Simulation
-    task_name: TaskName
-    folder_name: str = "default"
-    task_id: TaskId = None
-    callback_url: str = None
+    task_name: TaskName = pd.Field(..., title="Task Name", description="Unique name of the task.")
+
+    folder_name: str = pd.Field(
+        "default", title="Folder Name", description="Name of folder to store task on web UI."
+    )
+
+    task_id: TaskId = pd.Field(
+        None,
+        title="Task Id",
+        description="Task ID number, set when the task is uploaded, leave as None.",
+    )
+
+    callback_url: str = pd.Field(
+        None,
+        title="Callback URL",
+        description="Http PUT url to receive simulation finish event. "
+        "The body content is a json file with fields "
+        "``{'id', 'status', 'name', 'workUnit', 'solverVersion'}``.",
+    )
 
     def run(
         self, path: str = DEFAULT_DATA_PATH, normalize_index: Optional[int] = 0
@@ -56,7 +62,7 @@ class Job(WebContainer):
         normalize_index : int = 0
             If specified, normalizes the frequency-domain data by the amplitude spectrum of the
             source corresponding to ``simulation.sources[normalize_index]``.
-            This occurs when the data is loaded into a :class:`SimulationData` object.
+            This occurs when the data is loaded into a :class:`.SimulationData` object.
             To turn off normalization, set ``normalize_index`` to ``None``.
 
         Returns
@@ -130,20 +136,7 @@ class Job(WebContainer):
         To load the output of completed simulation into :class:`.SimulationData`objets,
         call :meth:`Job.load`.
         """
-
-        status = self.status
-        console = Console()
-
-        with console.status(f"[bold green]Working on '{self.task_name}'...", spinner="runner") as _:
-
-            while status not in ("success", "error", "diverged", "deleted", "draft"):
-                new_status = self.status
-                if new_status == "visualize":
-                    new_status = "success"
-                if new_status != status:
-                    console.log(f"status = {new_status}")
-                    status = new_status
-                time.sleep(web.REFRESH_TIME)
+        web.monitor(self.task_id)
 
     def download(self, path: str = DEFAULT_DATA_PATH) -> None:
         """Download results of simulation.
@@ -172,7 +165,7 @@ class Job(WebContainer):
         normalize_index : int = 0
             If specified, normalizes the frequency-domain data by the amplitude spectrum of the
             source corresponding to ``simulation.sources[normalize_index]``.
-            This occurs when the data is loaded into a :class:`SimulationData` object.
+            This occurs when the data is loaded into a :class:`.SimulationData` object.
             To turn off normalization, set ``normalize_index`` to ``None``.
 
         Returns
@@ -192,56 +185,106 @@ class Job(WebContainer):
         self.task_id = None
 
 
+class BatchData(Tidy3dBaseModel):
+    """Holds a collection of :class:`.SimulationData` returned by :class:`.Batch`."""
+
+    task_paths: Dict[TaskName, str] = pd.Field(
+        ...,
+        title="Data Paths",
+        description="Mapping of task_name to path to corresponding data for each task in batch.",
+    )
+
+    task_ids: Dict[TaskName, str] = pd.Field(
+        ..., title="Task IDs", description="Mapping of task_name to task_id for each task in batch."
+    )
+
+    normalize_index: Optional[int] = pd.Field(
+        0,
+        title="Source Normalization Index",
+        description="If specified, normalizes the frequency-domain data by the amplitude "
+        "spectrum of the source corresponding to ``simulation.sources[normalize_index]``. "
+        "If ``None``, does not normalize.",
+    )
+
+    def load_sim_data(self, task_name: str) -> SimulationData:
+        """Load a :class:`.SimulationData` from file by task name."""
+        task_data_path = self.task_paths[task_name]
+        task_id = self.task_ids[task_name]
+        return web.load(
+            task_id=task_id,
+            path=task_data_path,
+            normalize_index=self.normalize_index,
+            replace_existing=False,
+        )
+
+    def items(self) -> Tuple[TaskName, SimulationData]:
+        """Iterate through the :class:`.SimulationData` for each task_name."""
+        for task_name in self.task_paths.keys():
+            yield task_name, self.load_sim_data(task_name)
+
+    def __getitem__(self, task_name: TaskName) -> SimulationData:
+        """Get the :class:`.SimulationData` for a given ``task_name``."""
+        return self.load_sim_data(task_name)
+
+
 class Batch(WebContainer):
-    """Interface for submitting several :class:`.Simulation` objects to sever.
+    """Interface for submitting several :class:`.Simulation` objects to sever."""
 
-    Parameters
-    ----------
-    simulations : Dict[str, :class:`.Simulation`]
-        Mapping of task name to :class:`.Simulation` objects.
-    folder_name : ``str`` = './'
-        Name of folder to store member of each batch on web UI.
-    """
+    simulations: Dict[TaskName, Simulation] = pd.Field(
+        ...,
+        title="Simulations",
+        description="Mapping of task names to Simulations to run as a batch.",
+    )
 
-    simulations: Dict[TaskName, Simulation]
-    jobs: Dict[TaskName, Job] = None
-    folder_name: str = "default"
+    jobs: Dict[TaskName, Job] = pd.Field(
+        None,
+        title="Simulations",
+        description="Mapping of task names to individual Job object for each task in the batch. "
+        "Set by ``Batch.upload``, leave as None.",
+    )
 
-    def run(self, path_dir: str = DEFAULT_DATA_DIR):
+    folder_name: str = pd.Field(
+        "default",
+        title="Folder Name",
+        description="Name of folder to store member of each batch on web UI.",
+    )
+
+    def run(
+        self, path_dir: str = DEFAULT_DATA_DIR, normalize_index: Optional[int] = 0
+    ) -> BatchData:
         """Upload and run each simulation in :class:`Batch`.
-        Returns generator that can be used to loop through data results.
 
         Parameters
         ----------
         path_dir : str
             Base directory where data will be downloaded, by default current working directory.
 
-        Yields
+        Returns
         ------
-        str, :class:`.SimulationData`
-            Yields the name of task
-            and its corresponding :class:`.SimulationData` at each iteration.
+        :class:`BatchData`
+            Contains the :class:`.SimulationData` of each :class:`.Simulation` in :class:`Batch`.
 
         Note
         ----
         A typical usage might look like:
 
-        >>> batch_results = batch.run()
-        >>> for task_name, sim_data in batch_results:
+        >>> batch_data = batch.run()
+        >>> for task_name, sim_data in batch_data.items():
         ...     # do something with data.
 
-        Note that because ``batch_results`` is a generator, only the current iteration of
-        :class:`.SimulationData` is stored in memory at a time.
+        ``bach_data`` does not store all of the :class:`.SimulationData` objects in memory,
+        rather it iterates over the task names
+        and loads the corresponding :class:`.SimulationData` from file.
+        If no file exists for that task, it downloads it.
         """
 
         self.upload()
         self.start()
         self.monitor()
-        self.download(path_dir=path_dir)
-        return self.items()
+        return self.load(path_dir=path_dir, normalize_index=normalize_index)
 
     def upload(self) -> None:
-        """Create a series of tasks in the :class:`Batch` and upload them to server.
+        """Create a series of tasks in the :class:`.Batch` and upload them to server.
 
         Note
         ----
@@ -303,6 +346,7 @@ class Batch(WebContainer):
             return f"{task_name}: status = {status}"
 
         run_statuses = [
+            "draft",
             "queued",
             "preprocess",
             "queued_solver",
@@ -313,34 +357,29 @@ class Batch(WebContainer):
         ]
         end_statuses = ("success", "error", "diverged", "deleted", "draft")
 
-        pbar_tasks = {}
-        with Progress() as progress:
+        console = Console()
+        console.log("Started working on Batch.")
+
+        with Progress(console=console) as progress:
+
+            # create progressbars
+            pbar_tasks = {}
             for task_name, job in self.jobs.items():
                 status = job.status
                 description = pbar_description(task_name, status)
                 pbar = progress.add_task(description, total=len(run_statuses) - 1)
                 pbar_tasks[task_name] = pbar
 
-            statuses = [job.status for _, job in self.jobs.items()]
-            while any(s not in end_statuses for s in statuses):
-                for status_index, (task_name, job) in enumerate(self.jobs.items()):
-                    current_status = statuses[status_index]
-                    new_status = job.status
-                    if new_status == "visualize":
-                        new_status = "success"
-                    if new_status != current_status:
-                        completed = run_statuses.index(new_status)
-                        pbar = pbar_tasks[task_name]
-                        description = pbar_description(task_name, new_status)
-                        progress.update(pbar, description=description, completed=completed)
-                        statuses[status_index] = new_status
-                statuses = [job.status for _, job in self.jobs.items()]
+            while any(job.status not in end_statuses for job in self.jobs.values()):
+                for task_name, job in self.jobs.items():
+                    pbar = pbar_tasks[task_name]
+                    status = job.status
+                    description = pbar_description(task_name, status)
+                    completed = run_statuses.index(status)
+                    progress.update(pbar, description=description, completed=completed)
                 time.sleep(web.REFRESH_TIME)
 
-            for task_name, job in self.jobs.items():
-                pbar = pbar_tasks[task_name]
-                description = pbar_description(task_name, job.status)
-                progress.update(pbar, description=description, completed=len(run_statuses))
+        console.log("Batch complete.")
 
     @staticmethod
     def _job_data_path(task_id: TaskId, path_dir: str = DEFAULT_DATA_DIR):
@@ -376,13 +415,13 @@ class Batch(WebContainer):
 
         """
 
-        for task_name, job in self.jobs.items():
-            job_path = self._job_data_path(task_name, path_dir)
+        for job in self.jobs.values():
+            job_path = self._job_data_path(task_id=job.task_id, path_dir=path_dir)
             job.download(path=job_path)
 
     def load(
         self, path_dir: str = DEFAULT_DATA_DIR, normalize_index: Optional[int] = 0
-    ) -> Dict[TaskName, SimulationData]:
+    ) -> BatchData:
         """Download results and load them into :class:`.SimulationData` object.
 
         Parameters
@@ -391,49 +430,20 @@ class Batch(WebContainer):
             Base directory where data will be downloaded, by default current working directory.
 
         Returns
-        -------
-        Dict[str, :class:`.SimulationData`]
-            Dictionary mapping task names to :class:`.SimulationData` for :class:`Batch`.
-
-        Note
-        ----
-        This will return a dictionary of :class:`.SimulationData` objects,
-        each of which can hold a large amount of data.
-        If many simulations or large amounts of data,
-        use ``for task_name, sim_data in Batch.items():``
-        to instead loop through :class:`.SimulationData` objects and only store
-        current iteration in memory.
+        ------
+        :class:`BatchData`
+            Contains the :class:`.SimulationData` of each :class:`.Simulation` in :class:`Batch`.
         """
-        sim_data_dir = {}
-        # self.download(path_dir=path_dir)
+        task_paths = {}
+        task_ids = {}
         for task_name, job in self.jobs.items():
-            job_path = self._job_data_path(task_id=job.task_id, path_dir=path_dir)
-            sim_data = job.load(path=job_path, normalize_index=normalize_index)
-            sim_data_dir[task_name] = sim_data
-        return sim_data_dir
+            task_paths[task_name] = self._job_data_path(task_id=job.task_id, path_dir=path_dir)
+            task_ids[task_name] = self.jobs[task_name].task_id
+
+        return BatchData(task_paths=task_paths, task_ids=task_ids, normalize_index=normalize_index)
 
     def delete(self):
         """Delete server-side data associated with each task in the batch."""
         for _, job in self.jobs.items():
             job.delete()
             self.jobs = None
-
-    def items(self, path_dir: str = DEFAULT_DATA_DIR) -> Generator:
-        """Generates :class:`.SimulationData` for batch.
-        Used like: ``for task_name, sim_data in batch.items(): do something``.
-
-        Parameters
-        ----------
-        path_dir : str = './'
-            Base directory where data will be downloaded, by default current working directory.
-
-        Yields
-        ------
-        str, :class:`.SimulationData`
-            Yields the name of task
-            and its corresponding :class:`.SimulationData` at each iteration.
-        """
-        for task_name, job in self.jobs.items():
-            job_path = self._job_data_path(task_id=job.task_id, path_dir=path_dir)
-            sim_data = job.load(path=job_path)
-            yield task_name, sim_data
