@@ -1,25 +1,26 @@
 """Provides lowest level, user-facing interface to server."""
 
+import json
+import logging
 import os
 import time
-import json
+from datetime import datetime, timedelta
 from typing import List, Dict, Optional
-from datetime import datetime
-import logging
 
 import requests
+from dateutil import parser
 from rich.console import Console
 from rich.progress import Progress
 
-from .config import DEFAULT_CONFIG as Config
-from .s3utils import get_s3_user, DownloadProgress
-from .auth import requires_auth
-from .task import TaskId, TaskInfo
 from . import httputils as http
-from ..components.simulation import Simulation
+from .config import DEFAULT_CONFIG
+from .s3utils import get_s3_user, DownloadProgress
+from .task import TaskId, TaskInfo, Folder
 from ..components.data import SimulationData
+from ..components.simulation import Simulation
 from ..components.types import Literal
 from ..log import log, WebError
+from ..version import __version__
 
 REFRESH_TIME = 0.3
 TOTAL_DOTS = 3
@@ -71,7 +72,7 @@ def run(  # pylint:disable=too-many-arguments
     return load(task_id=task_id, path=path, normalize_index=normalize_index)
 
 
-def upload(
+def upload(  # pylint:disable=too-many-locals,too-many-arguments
     simulation: Simulation, task_name: str, folder_name: str = "default", callback_url: str = None
 ) -> TaskId:
     """Upload simulation to server, but do not start running :class:`.Simulation`.
@@ -98,20 +99,50 @@ def upload(
     To start the simulation running, must call :meth:`start` after uploaded.
     """
 
-    task_id = _upload_task(
-        simulation=simulation,
-        task_name=task_name,
-        folder_name=folder_name,
-        callback_url=callback_url,
-    )
+    simulation.validate_pre_upload()
 
+    json_string = simulation._json_string()  # pylint:disable=protected-access
+    data = {
+        "taskName": task_name,
+        "callbackUrl": callback_url,
+    }
+    folder = _query_or_create_folder(folder_name)
+    method = f"tidy3d/projects/{folder.projectId}/tasks"
+
+    log.debug("Creating task.")
+    try:
+        task = http.post(method=method, data=data)
+        task_id = task["taskId"]
+    except requests.exceptions.HTTPError as e:
+        error_json = json.loads(e.response.text)
+        raise WebError(error_json["error"]) from e
     # log the task_id so users can copy and paste it from STDOUT / file if the need it later.
     log.info(f"Uploaded task '{task_name}' with task_id '{task_id}'.")
 
+    # upload the file to s3
+    log.debug("Uploading the json file")
+    client, bucket, user_id = get_s3_user()
+    key = f"users/{user_id}/{task_id}/simulation.json"
+    log.debug(f"json = {json_string}")
+    resp = client.put_object(
+        Body=json_string,
+        Bucket=bucket,
+        Key=key,
+    )
+    if resp.get("ResponseMetadata") and resp.get("ResponseMetadata").get("HTTPStatusCode") != 200:
+        raise WebError("Upload file to 3S failed, please check your network or try it later.")
+
+    # log the url for the task in the web UI
+    log.debug(f"{DEFAULT_CONFIG.website_endpoint}/folders/{folder.projectId}/tasks/{task_id}")
+
+    # log the maximum flex unit cost
+    max_cost = estimate_cost(task_id)
+    if max_cost is not None:
+        log.info(f"Maximum flex unit cost: {max_cost:1.2f}")
+
     return task_id
 
 
-@requires_auth
 def get_info(task_id: TaskId) -> TaskInfo:
     """Return information about a task.
 
@@ -125,34 +156,42 @@ def get_info(task_id: TaskId) -> TaskInfo:
     :class:`TaskInfo`
         Object containing information about status, size, credits of task.
     """
-    method = f"fdtd/task/{task_id}"
+    method = f"tidy3d/tasks/{task_id}/detail"
     info_dict = http.get(method)
     if info_dict is None:
         raise WebError(f"task {task_id} not found, unable to load info.")
     return TaskInfo(**info_dict)
 
 
-@requires_auth
 def start(task_id: TaskId) -> None:
     """Start running the simulation associated with task.
 
     Parameters
     ----------
+
     task_id : str
         Unique identifier of task on server.  Returned by :meth:`upload`.
-
+    solver_version : str
+        Supply or override a specific solver version to the task.
     Note
     ----
     To monitor progress, can call :meth:`monitor` after starting simulation.
     """
-    task = get_info(task_id)
-    folder_name = task.folderId
-    method = f"fdtd/model/{folder_name}/task/{task_id}"
-    task.status = "queued"
-    http.put(method, data=task.dict())
+    method = f"tidy3d/tasks/{task_id}/submit"
+    data = {}
+
+    # do not pass protocol version if mapping is missing or needs an override.
+    if DEFAULT_CONFIG.solver_version:
+        data["solverVersion"] = DEFAULT_CONFIG.solver_version
+    else:
+        data["protocolVersion"] = __version__
+
+    if DEFAULT_CONFIG.worker_group:
+        data["workerGroup"] = DEFAULT_CONFIG.worker_group
+
+    http.post(method, data=data)
 
 
-@requires_auth
 def get_run_info(task_id: TaskId):
     """Gets the % done and field_decay for a running task.
 
@@ -181,7 +220,6 @@ def get_run_info(task_id: TaskId):
         return None, None
 
 
-@requires_auth
 def monitor(task_id: TaskId) -> None:
     """Print the real time task progress until completion.
 
@@ -213,7 +251,7 @@ def monitor(task_id: TaskId) -> None:
 
     # already done
     if get_status() in break_statuses:
-        console.log(f"status = {get_status()}")
+        log.info(f"status = {get_status()}")
         return
 
     # preprocessing
@@ -222,16 +260,16 @@ def monitor(task_id: TaskId) -> None:
             if status != get_status():
                 status = get_status()
                 if status != "running":
-                    console.log(f"status = {status}")
+                    log.info(f"status = {status}")
             time.sleep(REFRESH_TIME)
 
     # startup phase where run info is not available
-    console.log("starting up solver")
+    log.info("starting up solver")
     while get_run_info(task_id)[0] is None and get_status() == "running":
         time.sleep(REFRESH_TIME)
 
     # phase where run % info is available
-    console.log("running solver")
+    log.info("running solver")
     with Progress(console=console) as progress:
         pbar_pd = progress.add_task("% done", total=100)
         perc_done, _ = get_run_info(task_id)
@@ -241,7 +279,7 @@ def monitor(task_id: TaskId) -> None:
             progress.update(pbar_pd, completed=perc_done, description=new_description)
             time.sleep(1.0)
         if perc_done is not None and perc_done < 100:
-            console.log("early shutoff detected, exiting.")
+            log.info("early shutoff detected, exiting.")
         else:
             progress.update(pbar_pd, completed=100)
 
@@ -250,12 +288,12 @@ def monitor(task_id: TaskId) -> None:
         while get_status() not in break_statuses:
             if status != get_status():
                 status = get_status()
-                console.log(f"status = {status}")
+                log.info(f"status = {status}")
             time.sleep(REFRESH_TIME)
 
     # final status (if diffrent from the last printed status)
     if status != get_status():
-        console.log(f"status = {get_status()}")
+        log.info(f"status = {get_status()}")
 
 
 def download(task_id: TaskId, path: str = "simulation_data.hdf5") -> None:
@@ -275,7 +313,7 @@ def download(task_id: TaskId, path: str = "simulation_data.hdf5") -> None:
 
     # TODO: it should be possible to load "diverged" simulations
     task_info = get_info(task_id)
-    if task_info.status in ("error", "deleted", "draft"):
+    if task_info.status in ("error", "deleted"):
         raise WebError(f"can't download task '{task_id}', status = '{task_info.status}'")
 
     directory, _ = os.path.split(path)
@@ -317,7 +355,7 @@ def load(
         download(task_id=task_id, path=path)
 
     log.info(f"loading SimulationData from {path}")
-    sim_data = SimulationData.from_file(path)
+    sim_data = SimulationData.from_file(path, normalize_index=normalize_index)
 
     final_decay_value = sim_data.final_decay_value
     shutoff_value = sim_data.simulation.shutoff
@@ -328,13 +366,9 @@ def load(
             "Consider simulation again with large run_time duration for more accurate results."
         )
 
-    if normalize_index is not None:
-        return sim_data.normalize(normalize_index=normalize_index)
-
     return sim_data
 
 
-@requires_auth
 def delete(task_id: TaskId) -> TaskInfo:
     """Delete server-side data associated with task.
 
@@ -349,20 +383,22 @@ def delete(task_id: TaskId) -> TaskInfo:
         Object containing information about status, size, credits of task.
     """
 
-    method = f"fdtd/task/{str(task_id)}"
+    method = f"tidy3d/tasks/{str(task_id)}"
     return http.delete(method)
 
 
-@requires_auth
-def delete_old(days_old: int = 100, folder: str = None) -> int:
+def delete_old(
+    days_old: int = 100,
+    folder: str = "default",
+) -> int:
     """Delete all tasks older than a given amount of days.
 
     Parameters
     ----------
+    folder : str
+        Only allowed to delete in one folder at a time.
     days_old : int = 100
         Minimum number of days since the task creation.
-    folder : str = None
-        If None, all folders are purged.
 
     Returns
     -------
@@ -370,25 +406,23 @@ def delete_old(days_old: int = 100, folder: str = None) -> int:
         Total number of tasks deleted.
     """
 
-    tasks = http.get("fdtd/models")
+    folder = _query_or_create_folder(folder)
+    tasks = http.get(f"tidy3d/projects/{folder.projectId}/tasks")
     count = 0
-    for pfolder in tasks:
-        if pfolder["name"] == folder or folder is None:
-            for task in pfolder["children"]:
-                if task["status"] == "deleted":
-                    continue
-                stime_str = task["submitTime"]
-                stime = datetime.strptime(stime_str, "%Y:%m:%d:%H:%M:%S")
-                days_elapsed = (datetime.utcnow() - stime).days
-                if days_elapsed >= days_old:
-                    delete(task["taskId"])
-                    count += 1
+    for task in tasks:
+        stime_str = task["createdAt"]
+        stime = parser.parse(stime_str)
+        cutoff_date = datetime.now(stime.tzinfo) - timedelta(days=days_old)
+        if stime < cutoff_date:
+            delete(task["taskId"])
+            count += 1
 
     return count
 
 
-@requires_auth
-def get_tasks(num_tasks: int = None, order: Literal["new", "old"] = "new") -> List[Dict]:
+def get_tasks(
+    num_tasks: int = None, order: Literal["new", "old"] = "new", folder: str = "default"
+) -> List[Dict]:
     """Get a list with the metadata of the last ``num_tasks`` tasks.
 
     Parameters
@@ -397,24 +431,25 @@ def get_tasks(num_tasks: int = None, order: Literal["new", "old"] = "new") -> Li
         The number of tasks to return, or, if ``None``, return all.
     order : Literal["new", "old"] = "new"
         Return the tasks in order of newest-first or oldest-first.
+    folder: str = "default"
+        Folder from which to get the tasks.
     """
-
-    tasks = http.get("fdtd/models")
+    folder = _query_or_create_folder(folder)
+    tasks = http.get(f"tidy3d/projects/{folder.projectId}/tasks")
     store_dict = {
         "submit_time": [],
         "status": [],
         "task_name": [],
         "task_id": [],
     }
-    for pfolder in tasks:
-        for task in pfolder["children"]:
-            try:
-                store_dict["submit_time"].append(task["submitTime"])
-                store_dict["status"].append(task["status"])
-                store_dict["task_name"].append(task["taskName"])
-                store_dict["task_id"].append(task["taskId"])
-            except KeyError:
-                logging.warning(f"Error with task {task['taskId']}, skipping.")
+    for task in tasks:
+        try:
+            store_dict["submit_time"].append(task["createdAt"])
+            store_dict["status"].append(task["status"])
+            store_dict["task_name"].append(task["taskName"])
+            store_dict["task_id"].append(task["taskId"])
+        except KeyError:
+            logging.warning(f"Error with task {task['taskId']}, skipping.")
 
     sort_inds = sorted(
         range(len(store_dict["submit_time"])),
@@ -432,64 +467,46 @@ def get_tasks(num_tasks: int = None, order: Literal["new", "old"] = "new") -> Li
     return out_dict
 
 
-@requires_auth
-def _upload_task(  # pylint:disable=too-many-locals,too-many-arguments
-    simulation: Simulation,
-    task_name: str,
-    folder_name: str = "default",
-    solver_version: str = Config.solver_version,
-    worker_group: str = Config.worker_group,
-    callback_url: str = None,
-) -> TaskId:
-    """upload with all kwargs exposed"""
+def estimate_cost(task_id: str) -> float:
+    """Compute the maximum flex unit charge for a given task, assuming the simulation runs for
+    the full ``run_time``. If early shut-off is triggered, the cost is adjusted proporionately.
+    """
 
-    simulation.validate_pre_upload()
+    method = f"tidy3d/tasks/{task_id}/metadata"
+    data = {}
 
-    json_string = simulation._json_string()  # pylint:disable=protected-access
-    data = {
-        "status": "uploading",
-        "solverVersion": solver_version,
-        "taskName": task_name,
-        "workerGroup": worker_group,
-        "callbackUrl": callback_url,
-    }
+    # do not pass protocol version if mapping is missing or needs an override.
+    if DEFAULT_CONFIG.solver_version:
+        data["solverVersion"] = DEFAULT_CONFIG.solver_version
+    else:
+        data["protocolVersion"] = __version__
 
-    method = f"fdtd/model/{folder_name}/task"
-
-    log.debug("Creating task.")
     try:
-        task = http.post(method=method, data=data)
-        task_id = task["taskId"]
-    except requests.exceptions.HTTPError as e:
-        error_json = json.loads(e.response.text)
-        raise WebError(error_json["error"]) from e
+        resp = http.post(method, data=data)
+    except Exception:  # pylint:disable=broad-except
+        log.warning("Could not get estimated cost!")
+        return None
 
-    # upload the file to s3
-    log.debug("Uploading the json file")
+    return resp.get("flex_unit")
 
-    client, bucket, user_id = get_s3_user()
 
-    key = f"users/{user_id}/{task_id}/simulation.json"
+def _query_or_create_folder(folder_name) -> Folder:
+    log.debug("query folder")
+    method = f"tidy3d/project?projectName={folder_name}"
 
-    # size_bytes = len(json_string.encode('utf-8'))
-    # TODO: add progressbar, with put_object, no callback, so no real need.
-    # with Progress() as progress:
-    # upload_progress = UploadProgress(size_bytes, progress)
-    log.debug(f"json = {json_string}")
-    client.put_object(
-        Body=json_string,
-        Bucket=bucket,
-        Key=key,
-        # Callback=upload_progress.report
-    )
+    try:
+        resp = http.get(method)
+        if resp is None:
+            log.debug("folder not found, create one.")
+            method = "tidy3d/projects"
+            resp = http.post(method, data={"projectName": folder_name})
+        folder = Folder(**resp)
+    except Exception as e:  # pylint:disable=broad-except
+        raise WebError("Could not create task folder") from e
 
-    task["status"] = "draft"
-    http.put(f"{method}/{task_id}", data=task)
-
-    return task_id
+    return folder
 
 
-@requires_auth
 def _download_file(task_id: TaskId, fname: str, path: str) -> None:
     """Download a specific file from server.
 
